/*
    MartyPC
    https://github.com/dbalsom/martypc

    Copyright 2022-2023 Daniel Balsom

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the “Software”),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    --------------------------------------------------------------------------

    egui::color_swatch.rs

    Implements a custom control that displays a specified color. Intended to
    display palettte register entries.

*/

use crate::egui::GuiState;
use egui::*;

impl GuiState {

    pub fn color_swatch(ui: &mut Ui, color: Color32, open: bool) -> Response {
        let size = ui.spacing().interact_size;
        let size = egui::Vec2 { x: size.y, y: size.y}; // Make square
        let (rect, response) = ui.allocate_exact_size(size, Sense { click: false, drag: false, focusable: false});
        //response.widget_info(|| WidgetInfo::new(WidgetType::ColorButton));

        ui.spacing_mut().item_spacing = egui::vec2(0.0, 0.0);
        ui.spacing_mut().button_padding = egui::vec2(0.0, 0.0);

        if ui.is_rect_visible(rect) {
            let visuals = if open {
                &ui.visuals().widgets.open
            } else {
                ui.style().interact(&response)
            };
            //let rect = rect.expand(visuals.expansion);

            //painter.rect_filled(rect, 0.0, color);

            let rounding = visuals.rounding.at_most(2.0);

            ui.painter().rect_filled(rect, rounding, color);
            ui.painter().rect_stroke(rect, rounding, (2.0, visuals.bg_fill)); // fill is intentional, because default style has no border
        }

        response
    }
}